==> blueprint-handler-0.1.10.jar: blueprint/error.clj <==
(ns blueprint.error
  (:require [exoscale.ex :as ex]
            [clojure.tools.logging :as log]))

(defn ex->response*
  "Format user-exposable exceptions"
  [e defaults]
  (let [status (:http/status defaults 500)]
    {:status status
     :body (ex-message e)}))

(defmulti ex->response
  "Controls how exception response will happen.
  If the exception is a `:exoscale.ex/user-exposable` it will be
  dispatched normally, otherwise it will jump to `::ex/unexposable`
  implementation"
  (fn [e]
    (let [t (some-> e ex-data :type)]
      (if (ex/type? e ::ex/user-exposable)
        t
        ::ex/unexposable)))
  :hierarchy exoscale.ex/hierarchy)

(defmethod ex->response ::ex/incorrect [e] (ex->response* e {:http/status 400}))
(defmethod ex->response ::ex/not-found [e] (ex->response* e {:http/status 404}))
(defmethod ex->response ::ex/invalid-spec [e] (ex->response* e {:http/status 400}))
(defmethod ex->response ::ex/fault [e] (ex->response* e {:http/status 500}))
(defmethod ex->response ::ex/forbidden [e] (ex->response* e {:http/status 403}))
(defmethod ex->response ::ex/interrupted [e] (ex->response* e {:http/status 206}))
(defmethod ex->response ::ex/unsupported [e] (ex->response* e {:http/status 405}))
(defmethod ex->response ::ex/conflict [e] (ex->response* e {:http/status 409}))
(defmethod ex->response ::ex/unavailable [e] (ex->response* e {:http/status 504}))
(defmethod ex->response ::ex/busy [e] (ex->response* e {:http/status 503}))

(defn unexposable-handler
  "Install the method for the ::unexposable errors."
  [default-error-msg]
  (defmethod ex->response ::ex/unexposable [_]
    {:body default-error-msg
     :status 500}))

;; That would only be called if exception descends from ::ex/user-exposable
(defmethod ex->response :default [ex]
  (ex->response* ex {:http/status 500}))

(defmulti ex->log
  "Controls how exception logging will happen if at all.
  It will dispatch the call to the defmethods via the
  exception `:type`, defaulting to generic logging."
  (fn [e ctx]
    (some-> e ex-data :type))
  :hierarchy exoscale.ex/hierarchy)

(defn find-message
  "Finds most appropriate message within exception stack for
  logging/reporting purposes. Tries to avoid the default
  `default-error-msg`"
  [e default-error-msg]
  (let [msg (ex-message e)]
    (if (and (some? msg)
             (not= default-error-msg msg))
      msg
      (or (some-> e ex-cause find-message)
          default-error-msg))))

(defmethod ex->log :exoscale.ex/no-logging [_ _] nil)

(defmethod ex->log :default
  [e ctx]
  (let [request-id (:request-id ctx)]
    (log/error e
               request-id
               (find-message e (:default-error-msg ctx))
               (pr-str (ex-data e)))))

(defn not-found
  ([]
   (not-found "not found"))
  ([msg]
   (ex/ex-info msg [::not-found [::ex/not-found ::ex/user-exposable]])))

;; interceptors

(def exception-logger
  {:name ::exception-logger
   :error (fn [ctx e]
            (ex->log e ctx)
            (throw e))})

(def exception-response
  {:name ::exception-response
   :error (fn [ctx e]
            (assoc ctx :response (ex->response e)))})

==> blueprint-handler-0.1.12.jar: blueprint/error.clj <==
(ns blueprint.error
  (:require [exoscale.ex :as ex]
            [clojure.tools.logging :as log]))

(defn ex->response*
  "Format user-exposable exceptions"
  [e defaults]
  (let [status (:http/status defaults 500)]
    {:status status
     :body (ex-message e)}))

(defmulti ex->response
  "Controls how exception response will happen.
  If the exception is a `:exoscale.ex/user-exposable` it will be
  dispatched normally, otherwise it will jump to `::ex/unexposable`
  implementation"
  (fn [e]
    (let [t (some-> e ex-data :type)]
      (if (ex/type? e ::ex/user-exposable)
        t
        ::ex/unexposable)))
  :hierarchy exoscale.ex/hierarchy)

(defmethod ex->response ::ex/incorrect [e] (ex->response* e {:http/status 400}))
(defmethod ex->response ::ex/not-found [e] (ex->response* e {:http/status 404}))
(defmethod ex->response ::ex/invalid-spec [e] (ex->response* e {:http/status 400}))
(defmethod ex->response ::ex/fault [e] (ex->response* e {:http/status 500}))
(defmethod ex->response ::ex/forbidden [e] (ex->response* e {:http/status 403}))
(defmethod ex->response ::ex/interrupted [e] (ex->response* e {:http/status 206}))
(defmethod ex->response ::ex/unsupported [e] (ex->response* e {:http/status 405}))
(defmethod ex->response ::ex/conflict [e] (ex->response* e {:http/status 409}))
(defmethod ex->response ::ex/unavailable [e] (ex->response* e {:http/status 504}))
(defmethod ex->response ::ex/busy [e] (ex->response* e {:http/status 503}))

(defn unexposable-handler
  "Install the method for the ::unexposable errors."
  [default-error-msg]
  (defmethod ex->response ::ex/unexposable [_]
    {:body default-error-msg
     :status 500}))

;; That would only be called if exception descends from ::ex/user-exposable
(defmethod ex->response :default [ex]
  (ex->response* ex {:http/status 500}))

(defmulti ex->log
  "Controls how exception logging will happen if at all.
  It will dispatch the call to the defmethods via the
  exception `:type`, defaulting to generic logging."
  (fn [e ctx]
    (some-> e ex-data :type))
  :hierarchy exoscale.ex/hierarchy)

(defn find-message
  "Finds most appropriate message within exception stack for
  logging/reporting purposes. Tries to avoid the default
  `default-error-msg`"
  [e default-error-msg]
  (let [msg (ex-message e)]
    (if (and (some? msg)
             (not= default-error-msg msg))
      msg
      (or (some-> e ex-cause find-message)
          default-error-msg))))

(defmethod ex->log :exoscale.ex/no-logging [_ _] nil)

(defmethod ex->log :default
  [e ctx]
  (let [request-id (:request-id ctx)]
    (log/error e
               request-id
               (find-message e (:default-error-msg ctx))
               (pr-str (ex-data e)))))

(defn not-found
  ([]
   (not-found "not found"))
  ([msg]
   (ex/ex-info msg [::not-found [::ex/not-found ::ex/user-exposable]])))

(defn wrap-incorrect
  "A function which will wrap an exception with an exception that derives
  ::ex/incorrect and ::ex/user-exposable, while copying messages and ex-data"
  [cause]
  (ex/ex-info (ex-message cause)
              [::incorrect [::ex/incorrect ::ex/user-exposable]]
              (ex-data cause)
              cause))

;; interceptors

(def exception-logger
  {:name ::exception-logger
   :error (fn [ctx e]
            (ex->log e ctx)
            (throw e))})

(def exception-response
  {:name ::exception-response
   :error (fn [ctx e]
            (assoc ctx :response (ex->response e)))})
